@@ -49,39 +49,60 @@
 #define openconnect_has_oath_support() 0
 #endif
 
-#define NM_VPN_API_SUBJECT_TO_CHANGE
-
+#ifdef NM_OPENCONNECT_OLD
+#define NM_VPN_LIBNM_COMPAT
 #include <nm-vpn-plugin-ui-interface.h>
 #include <nm-setting-vpn.h>
 #include <nm-setting-connection.h>
 #include <nm-setting-ip4-config.h>
 
-#include "../src/nm-openconnect-service.h"
+#define nm_simple_connection_new nm_connection_new
+
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR                   NM_SETTING_VPN_ERROR
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY  NM_SETTING_VPN_ERROR_INVALID_PROPERTY
+
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR                   NM_SETTING_VPN_ERROR
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY  NM_SETTING_VPN_ERROR_INVALID_PROPERTY
+
+#else /* !NM_OPENCONNECT_OLD */
+
+#include <NetworkManager.h>
+
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR                   NM_CONNECTION_ERROR
+#define OPENCONNECT_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY  NM_CONNECTION_ERROR_INVALID_PROPERTY
+#endif
+
+#include "nm-openconnect-service-defines.h"
 #include "nm-openconnect.h"
 #include "auth-helpers.h"
 
 #define OPENCONNECT_PLUGIN_NAME    _("Cisco AnyConnect Compatible VPN (openconnect)")
 #define OPENCONNECT_PLUGIN_DESC    _("Compatible with Cisco AnyConnect SSL VPN.")
-#define OPENCONNECT_PLUGIN_SERVICE NM_DBUS_SERVICE_OPENCONNECT 
-
 
 /************** plugin class **************/
 
-static void openconnect_plugin_ui_interface_init (NMVpnPluginUiInterface *iface_class);
+enum {
+	PROP_0,
+	PROP_NAME,
+	PROP_DESC,
+	PROP_SERVICE
+};
 
-G_DEFINE_TYPE_EXTENDED (OpenconnectPluginUi, openconnect_plugin_ui, G_TYPE_OBJECT, 0,
-						G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_PLUGIN_UI_INTERFACE,
-											   openconnect_plugin_ui_interface_init))
+static void openconnect_editor_plugin_interface_init (NMVpnEditorPluginInterface *iface_class);
+
+G_DEFINE_TYPE_EXTENDED (OpenconnectEditorPlugin, openconnect_editor_plugin, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_EDITOR_PLUGIN,
+                                               openconnect_editor_plugin_interface_init))
 
 /************** UI widget class **************/
 
-static void openconnect_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface *iface_class);
+static void openconnect_editor_interface_init (NMVpnEditorInterface *iface_class);
 
-G_DEFINE_TYPE_EXTENDED (OpenconnectPluginUiWidget, openconnect_plugin_ui_widget, G_TYPE_OBJECT, 0,
-						G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_PLUGIN_UI_WIDGET_INTERFACE,
-											   openconnect_plugin_ui_widget_interface_init))
+G_DEFINE_TYPE_EXTENDED (OpenconnectEditor, openconnect_editor, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (NM_TYPE_VPN_EDITOR,
+                                               openconnect_editor_interface_init))
 
-#define OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OPENCONNECT_TYPE_PLUGIN_UI_WIDGET, OpenconnectPluginUiWidgetPrivate))
+#define OPENCONNECT_EDITOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), OPENCONNECT_TYPE_EDITOR, OpenconnectEditorPrivate))
 
 typedef struct {
 	GtkBuilder *builder;
@@ -89,7 +110,7 @@ typedef struct {
 	GtkSizeGroup *group;
 	GtkWindowGroup *window_group;
 	gboolean window_added;
-} OpenconnectPluginUiWidgetPrivate;
+} OpenconnectEditorPrivate;
 
 #define COL_AUTH_NAME 0
 #define COL_AUTH_PAGE 1
@@ -116,11 +137,11 @@ nm_openconnect_import_export_error_quark (void)
 }
 
 static NMConnection *
-import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
+import (NMVpnEditorPlugin *iface, const char *path, GError **error)
 {
 	NMConnection *connection;
 	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	NMSettingIP4Config *s_ip4;
 	GKeyFile *keyfile;
 	GKeyFileFlags flags;
@@ -139,12 +160,12 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 		return NULL;
 	}
 
-	connection = nm_connection_new ();
+	connection = nm_simple_connection_new ();
 	s_con = NM_SETTING_CONNECTION (nm_setting_connection_new ());
 	nm_connection_add_setting (connection, NM_SETTING (s_con));
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
-	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_OPENCONNECT, NULL);
+	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_VPN_SERVICE_TYPE_OPENCONNECT, NULL);
 	nm_connection_add_setting (connection, NM_SETTING (s_vpn));
 
 	s_ip4 = NM_SETTING_IP4_CONFIG (nm_setting_ip4_config_new ());
@@ -220,13 +241,13 @@ import (NMVpnPluginUiInterface *iface, const char *path, GError **error)
 }
 
 static gboolean
-export (NMVpnPluginUiInterface *iface,
+export (NMVpnEditorPlugin *iface,
         const char *path,
         NMConnection *connection,
         GError **error)
 {
 	NMSettingConnection *s_con;
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	const char *value;
 	const char *gateway = NULL;
 	const char *cacert = NULL;
@@ -250,9 +271,9 @@ export (NMVpnPluginUiInterface *iface,
 		return FALSE;
 	}
 
-	s_con = NM_SETTING_CONNECTION (nm_connection_get_setting (connection, NM_TYPE_SETTING_CONNECTION));
+	s_con = nm_connection_get_setting_connection (connection);
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 
 	value = nm_setting_vpn_get_data_item (s_vpn, NM_OPENCONNECT_KEY_GATEWAY);
 	if (value && strlen (value))
@@ -338,44 +359,10 @@ done:
 	return success;
 }
 
-GQuark
-openconnect_plugin_ui_error_quark (void)
-{
-	static GQuark error_quark = 0;
-
-	if (G_UNLIKELY (error_quark == 0))
-		error_quark = g_quark_from_static_string ("openconnect-plugin-ui-error-quark");
-
-	return error_quark;
-}
-
-/* This should really be standard. */
-#define ENUM_ENTRY(NAME, DESC) { NAME, "" #NAME "", DESC }
-
-GType
-openconnect_plugin_ui_error_get_type (void)
-{
-	static GType etype = 0;
-
-	if (etype == 0) {
-		static const GEnumValue values[] = {
-			/* Unknown error. */
-			ENUM_ENTRY (OPENCONNECT_PLUGIN_UI_ERROR_UNKNOWN, "UnknownError"),
-			/* The specified property was invalid. */
-			ENUM_ENTRY (OPENCONNECT_PLUGIN_UI_ERROR_INVALID_PROPERTY, "InvalidProperty"),
-			/* The specified property was missing and is required. */
-			ENUM_ENTRY (OPENCONNECT_PLUGIN_UI_ERROR_MISSING_PROPERTY, "MissingProperty"),
-			{ 0, 0, 0 }
-		};
-		etype = g_enum_register_static ("OpenconnectPluginUiError", values);
-	}
-	return etype;
-}
-
 static gboolean
-check_validity (OpenconnectPluginUiWidget *self, GError **error)
+check_validity (OpenconnectEditor *self, GError **error)
 {
-	OpenconnectPluginUiWidgetPrivate *priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	OpenconnectEditorPrivate *priv = OPENCONNECT_EDITOR_GET_PRIVATE (self);
 	GtkWidget *widget;
 	const char *str;
 
@@ -383,8 +370,8 @@ check_validity (OpenconnectPluginUiWidget *self, GError **error)
 	str = gtk_entry_get_text (GTK_ENTRY (widget));
 	if (!str || !strlen (str)) {
 		g_set_error (error,
-		             OPENCONNECT_PLUGIN_UI_ERROR,
-		             OPENCONNECT_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENCONNECT_EDITOR_PLUGIN_ERROR,
+		             OPENCONNECT_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENCONNECT_KEY_GATEWAY);
 		return FALSE;
 	}
@@ -395,8 +382,8 @@ check_validity (OpenconnectPluginUiWidget *self, GError **error)
 	if (str && str[0] &&
 		strncmp(str, "socks://", 8) && strncmp(str, "http://", 7)) {
 		g_set_error (error,
-		             OPENCONNECT_PLUGIN_UI_ERROR,
-		             OPENCONNECT_PLUGIN_UI_ERROR_INVALID_PROPERTY,
+		             OPENCONNECT_EDITOR_PLUGIN_ERROR,
+		             OPENCONNECT_EDITOR_PLUGIN_ERROR_INVALID_PROPERTY,
 		             NM_OPENCONNECT_KEY_PROXY);
 		return FALSE;
 	}
@@ -410,7 +397,7 @@ check_validity (OpenconnectPluginUiWidget *self, GError **error)
 static void
 stuff_changed_cb (GtkWidget *widget, gpointer user_data)
 {
-	g_signal_emit_by_name (OPENCONNECT_PLUGIN_UI_WIDGET (user_data), "changed");
+	g_signal_emit_by_name (OPENCONNECT_EDITOR (user_data), "changed");
 }
 
 static gboolean
@@ -447,9 +434,9 @@ init_token_mode_options (GtkComboBox *token_mode)
 }
 
 static gboolean
-init_token_ui (OpenconnectPluginUiWidget *self,
-				OpenconnectPluginUiWidgetPrivate *priv,
-				NMSettingVPN *s_vpn)
+init_token_ui (OpenconnectEditor *self,
+               OpenconnectEditorPrivate *priv,
+               NMSettingVpn *s_vpn)
 {
 	GtkWidget *widget;
 	GtkComboBox *token_mode;
@@ -512,14 +499,14 @@ init_token_ui (OpenconnectPluginUiWidget *self,
 }
 
 static gboolean
-init_plugin_ui (OpenconnectPluginUiWidget *self, NMConnection *connection, GError **error)
+init_editor_plugin (OpenconnectEditor *self, NMConnection *connection, GError **error)
 {
-	OpenconnectPluginUiWidgetPrivate *priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
-	NMSettingVPN *s_vpn;
+	OpenconnectEditorPrivate *priv = OPENCONNECT_EDITOR_GET_PRIVATE (self);
+	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
 	const char *value;
 
-	s_vpn = (NMSettingVPN *) nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN);
+	s_vpn = nm_connection_get_setting_vpn (connection);
 
 	priv->group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
 
@@ -584,22 +571,22 @@ init_plugin_ui (OpenconnectPluginUiWidget *self, NMConnection *connection, GErro
 }
 
 static GObject *
-get_widget (NMVpnPluginUiWidgetInterface *iface)
+get_widget (NMVpnEditor *iface)
 {
-	OpenconnectPluginUiWidget *self = OPENCONNECT_PLUGIN_UI_WIDGET (iface);
-	OpenconnectPluginUiWidgetPrivate *priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
+	OpenconnectEditor *self = OPENCONNECT_EDITOR (iface);
+	OpenconnectEditorPrivate *priv = OPENCONNECT_EDITOR_GET_PRIVATE (self);
 
 	return G_OBJECT (priv->widget);
 }
 
 static gboolean
-update_connection (NMVpnPluginUiWidgetInterface *iface,
+update_connection (NMVpnEditor *iface,
                    NMConnection *connection,
                    GError **error)
 {
-	OpenconnectPluginUiWidget *self = OPENCONNECT_PLUGIN_UI_WIDGET (iface);
-	OpenconnectPluginUiWidgetPrivate *priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (self);
-	NMSettingVPN *s_vpn;
+	OpenconnectEditor *self = OPENCONNECT_EDITOR (iface);
+	OpenconnectEditorPrivate *priv = OPENCONNECT_EDITOR_GET_PRIVATE (self);
+	NMSettingVpn *s_vpn;
 	GtkWidget *widget;
 	char *str;
 	GtkTreeModel *model;
@@ -610,7 +597,7 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 	const char *auth_type = NULL;
 
 	s_vpn = NM_SETTING_VPN (nm_setting_vpn_new ());
-	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_DBUS_SERVICE_OPENCONNECT, NULL);
+	g_object_set (s_vpn, NM_SETTING_VPN_SERVICE_TYPE, NM_VPN_SERVICE_TYPE_OPENCONNECT, NULL);
 
 	widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "gateway_entry"));
 	str = (char *) gtk_entry_get_text (GTK_ENTRY (widget));
@@ -671,21 +658,21 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 
 	/* These are different for every login session, and should not be stored */
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "gwcert",
-								 NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
+	                             NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "cookie",
-								 NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
+	                             NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "gateway",
-								 NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
+	                             NM_SETTING_SECRET_FLAG_NOT_SAVED, NULL);
 
 	/* These are purely internal data for the auth-dialog, and should be stored */
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "xmlconfig",
-								 NM_SETTING_SECRET_FLAG_NONE, NULL);
+	                             NM_SETTING_SECRET_FLAG_NONE, NULL);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "lasthost",
-								 NM_SETTING_SECRET_FLAG_NONE, NULL);
+	                             NM_SETTING_SECRET_FLAG_NONE, NULL);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "autoconnect",
-								 NM_SETTING_SECRET_FLAG_NONE, NULL);
+	                             NM_SETTING_SECRET_FLAG_NONE, NULL);
 	nm_setting_set_secret_flags (NM_SETTING (s_vpn), "certsigs",
-								 NM_SETTING_SECRET_FLAG_NONE, NULL);
+	                             NM_SETTING_SECRET_FLAG_NONE, NULL);
 	/* Note that the auth-dialog will also store "extra" secrets for form
 	   entries, depending on the arbitrary forms that we're offered by the
 	   server during authentication. We can't know about those in advance,
@@ -700,23 +687,23 @@ update_connection (NMVpnPluginUiWidgetInterface *iface,
 	return TRUE;
 }
 
-static NMVpnPluginUiWidgetInterface *
-nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
+static NMVpnEditor *
+nm_vpn_editor_interface_new (NMConnection *connection, GError **error)
 {
-	NMVpnPluginUiWidgetInterface *object;
-	OpenconnectPluginUiWidgetPrivate *priv;
+	NMVpnEditor *object;
+	OpenconnectEditorPrivate *priv;
 	char *ui_file;
 
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
 
-	object = NM_VPN_PLUGIN_UI_WIDGET_INTERFACE (g_object_new (OPENCONNECT_TYPE_PLUGIN_UI_WIDGET, NULL));
+	object = g_object_new (OPENCONNECT_TYPE_EDITOR, NULL);
 	if (!object) {
-		g_set_error (error, OPENCONNECT_PLUGIN_UI_ERROR, 0, "could not create openconnect object");
+		g_set_error (error, OPENCONNECT_EDITOR_PLUGIN_ERROR, 0, "could not create openconnect object");
 		return NULL;
 	}
 
-	priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (object);
+	priv = OPENCONNECT_EDITOR_GET_PRIVATE (object);
 
 	ui_file = g_strdup_printf ("%s/%s", UIDIR, "nm-openconnect-dialog.ui");
 	priv->builder = gtk_builder_new ();
@@ -727,7 +714,7 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 		g_warning ("Couldn't load builder file: %s",
 		           error && *error ? (*error)->message : "(unknown)");
 		g_clear_error (error);
-		g_set_error (error, OPENCONNECT_PLUGIN_UI_ERROR, 0,
+		g_set_error (error, OPENCONNECT_EDITOR_PLUGIN_ERROR, 0,
 		             "could not load required resources at %s", ui_file);
 		g_free (ui_file);
 		g_object_unref (object);
@@ -737,7 +724,7 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 
 	priv->widget = GTK_WIDGET (gtk_builder_get_object (priv->builder, "openconnect-vbox"));
 	if (!priv->widget) {
-		g_set_error (error, OPENCONNECT_PLUGIN_UI_ERROR, 0, "could not load UI widget");
+		g_set_error (error, OPENCONNECT_EDITOR_PLUGIN_ERROR, 0, "could not load UI widget");
 		g_object_unref (object);
 		return NULL;
 	}
@@ -745,7 +732,7 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 
 	priv->window_group = gtk_window_group_new ();
 
-	if (!init_plugin_ui (OPENCONNECT_PLUGIN_UI_WIDGET (object), connection, error)) {
+	if (!init_editor_plugin (OPENCONNECT_EDITOR (object), connection, error)) {
 		g_object_unref (object);
 		return NULL;
 	}
@@ -756,8 +743,8 @@ nm_vpn_plugin_ui_widget_interface_new (NMConnection *connection, GError **error)
 static void
 dispose (GObject *object)
 {
-	OpenconnectPluginUiWidget *plugin = OPENCONNECT_PLUGIN_UI_WIDGET (object);
-	OpenconnectPluginUiWidgetPrivate *priv = OPENCONNECT_PLUGIN_UI_WIDGET_GET_PRIVATE (plugin);
+	OpenconnectEditor *plugin = OPENCONNECT_EDITOR (object);
+	OpenconnectEditorPrivate *priv = OPENCONNECT_EDITOR_GET_PRIVATE (plugin);
 
 	if (priv->group)
 		g_object_unref (priv->group);
@@ -771,26 +758,26 @@ dispose (GObject *object)
 	if (priv->builder)
 		g_object_unref (priv->builder);
 
-	G_OBJECT_CLASS (openconnect_plugin_ui_widget_parent_class)->dispose (object);
+	G_OBJECT_CLASS (openconnect_editor_parent_class)->dispose (object);
 }
 
 static void
-openconnect_plugin_ui_widget_class_init (OpenconnectPluginUiWidgetClass *req_class)
+openconnect_editor_class_init (OpenconnectEditorClass *req_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
 
-	g_type_class_add_private (req_class, sizeof (OpenconnectPluginUiWidgetPrivate));
+	g_type_class_add_private (req_class, sizeof (OpenconnectEditorPrivate));
 
 	object_class->dispose = dispose;
 }
 
 static void
-openconnect_plugin_ui_widget_init (OpenconnectPluginUiWidget *plugin)
+openconnect_editor_init (OpenconnectEditor *plugin)
 {
 }
 
 static void
-openconnect_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface *iface_class)
+openconnect_editor_interface_init (NMVpnEditorInterface *iface_class)
 {
 	/* interface implementation */
 	iface_class->get_widget = get_widget;
@@ -798,32 +785,32 @@ openconnect_plugin_ui_widget_interface_init (NMVpnPluginUiWidgetInterface *iface
 }
 
 static guint32
-get_capabilities (NMVpnPluginUiInterface *iface)
+get_capabilities (NMVpnEditorPlugin *iface)
 {
-	return (NM_VPN_PLUGIN_UI_CAPABILITY_IMPORT |
-	        NM_VPN_PLUGIN_UI_CAPABILITY_EXPORT |
-	        NM_VPN_PLUGIN_UI_CAPABILITY_IPV6);
+	return (NM_VPN_EDITOR_PLUGIN_CAPABILITY_IMPORT |
+	        NM_VPN_EDITOR_PLUGIN_CAPABILITY_EXPORT |
+	        NM_VPN_EDITOR_PLUGIN_CAPABILITY_IPV6);
 }
 
-static NMVpnPluginUiWidgetInterface *
-ui_factory (NMVpnPluginUiInterface *iface, NMConnection *connection, GError **error)
+static NMVpnEditor *
+get_editor (NMVpnEditorPlugin *iface, NMConnection *connection, GError **error)
 {
-	return nm_vpn_plugin_ui_widget_interface_new (connection, error);
+	return nm_vpn_editor_interface_new (connection, error);
 }
 
 static void
 get_property (GObject *object, guint prop_id,
-			  GValue *value, GParamSpec *pspec)
+              GValue *value, GParamSpec *pspec)
 {
 	switch (prop_id) {
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_NAME:
+	case PROP_NAME:
 		g_value_set_string (value, OPENCONNECT_PLUGIN_NAME);
 		break;
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_DESC:
+	case PROP_DESC:
 		g_value_set_string (value, OPENCONNECT_PLUGIN_DESC);
 		break;
-	case NM_VPN_PLUGIN_UI_INTERFACE_PROP_SERVICE:
-		g_value_set_string (value, OPENCONNECT_PLUGIN_SERVICE);
+	case PROP_SERVICE:
+		g_value_set_string (value, NM_VPN_SERVICE_TYPE_OPENCONNECT);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -832,42 +819,42 @@ get_property (GObject *object, guint prop_id,
 }
 
 static void
-openconnect_plugin_ui_class_init (OpenconnectPluginUiClass *req_class)
+openconnect_editor_plugin_class_init (OpenconnectEditorPluginClass *req_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (req_class);
 
 	object_class->get_property = get_property;
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_NAME,
-									  NM_VPN_PLUGIN_UI_INTERFACE_NAME);
+	                                  PROP_NAME,
+	                                  NM_VPN_EDITOR_PLUGIN_NAME);
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_DESC,
-									  NM_VPN_PLUGIN_UI_INTERFACE_DESC);
+	                                  PROP_DESC,
+	                                  NM_VPN_EDITOR_PLUGIN_DESCRIPTION);
 
 	g_object_class_override_property (object_class,
-									  NM_VPN_PLUGIN_UI_INTERFACE_PROP_SERVICE,
-									  NM_VPN_PLUGIN_UI_INTERFACE_SERVICE);
+	                                  PROP_SERVICE,
+	                                  NM_VPN_EDITOR_PLUGIN_SERVICE);
 }
 
 static void
-openconnect_plugin_ui_init (OpenconnectPluginUi *plugin)
+openconnect_editor_plugin_init (OpenconnectEditorPlugin *plugin)
 {
 }
 
 static void
-openconnect_plugin_ui_interface_init (NMVpnPluginUiInterface *iface_class)
+openconnect_editor_plugin_interface_init (NMVpnEditorPluginInterface *iface_class)
 {
 	/* interface implementation */
-	iface_class->ui_factory = ui_factory;
+	iface_class->get_editor = get_editor;
 	iface_class->get_capabilities = get_capabilities;
 	iface_class->import_from_file = import;
 	iface_class->export_to_file = export;
 }
 
-G_MODULE_EXPORT NMVpnPluginUiInterface *
-nm_vpn_plugin_ui_factory (GError **error)
+G_MODULE_EXPORT NMVpnEditorPlugin *
+nm_vpn_editor_plugin_factory (GError **error)
 {
 	if (error)
 		g_return_val_if_fail (*error == NULL, NULL);
@@ -875,6 +862,6 @@ nm_vpn_plugin_ui_factory (GError **error)
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 
-	return NM_VPN_PLUGIN_UI_INTERFACE (g_object_new (OPENCONNECT_TYPE_PLUGIN_UI, NULL));
+	return g_object_new (OPENCONNECT_TYPE_EDITOR_PLUGIN, NULL);
 }
 
