@@ -43,7 +43,6 @@
 #include <locale.h>
 #include <glib/gi18n.h>
 
-#include <nm-setting-vpn.h>
 #include "nm-openconnect-service.h"
 #include "nm-utils.h"
 
@@ -51,14 +50,14 @@
 # define DIST_VERSION VERSION
 #endif
 
-G_DEFINE_TYPE (NMOPENCONNECTPlugin, nm_openconnect_plugin, NM_TYPE_VPN_PLUGIN)
+G_DEFINE_TYPE (NMOpenconnectPlugin, nm_openconnect_plugin, NM_TYPE_VPN_SERVICE_PLUGIN)
 
 typedef struct {
 	GPid pid;
 	char *tun_name;
-} NMOPENCONNECTPluginPrivate;
+} NMOpenconnectPluginPrivate;
 
-#define NM_OPENCONNECT_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_OPENCONNECT_PLUGIN, NMOPENCONNECTPluginPrivate))
+#define NM_OPENCONNECT_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), NM_TYPE_OPENCONNECT_PLUGIN, NMOpenconnectPluginPrivate))
 
 static const char *openconnect_binary_paths[] =
 {
@@ -71,7 +70,7 @@ static const char *openconnect_binary_paths[] =
 	NULL
 };
 
-#define NM_OPENCONNECT_HELPER_PATH		LIBEXECDIR"/nm-openconnect-service-openconnect-helper"
+#define NM_OPENCONNECT_HELPER_PATH LIBEXECDIR"/nm-openconnect-service-openconnect-helper"
 
 typedef struct {
 	const char *name;
@@ -184,7 +183,7 @@ validate_one_property (const char *key, const char *value, gpointer user_data)
 }
 
 static gboolean
-nm_openconnect_properties_validate (NMSettingVPN *s_vpn, GError **error)
+nm_openconnect_properties_validate (NMSettingVpn *s_vpn, GError **error)
 {
 	ValidateInfo info = { &valid_properties[0], error, FALSE };
 
@@ -202,7 +201,7 @@ nm_openconnect_properties_validate (NMSettingVPN *s_vpn, GError **error)
 }
 
 static gboolean
-nm_openconnect_secrets_validate (NMSettingVPN *s_vpn, GError **error)
+nm_openconnect_secrets_validate (NMSettingVpn *s_vpn, GError **error)
 {
 	ValidateInfo info = { &valid_secrets[0], error, FALSE };
 
@@ -311,8 +310,8 @@ static void openconnect_drop_child_privs(gpointer user_data)
 static void
 openconnect_watch_cb (GPid pid, gint status, gpointer user_data)
 {
-	NMOPENCONNECTPlugin *plugin = NM_OPENCONNECT_PLUGIN (user_data);
-	NMOPENCONNECTPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
+	NMOpenconnectPlugin *plugin = NM_OPENCONNECT_PLUGIN (user_data);
+	NMOpenconnectPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
 	guint error = 0;
 
 	if (WIFEXITED (status)) {
@@ -341,25 +340,24 @@ openconnect_watch_cb (GPid pid, gint status, gpointer user_data)
 	switch (error) {
 	case 2:
 		/* Couldn't log in due to bad user/pass */
-		nm_vpn_plugin_failure (NM_VPN_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_LOGIN_FAILED);
+		nm_vpn_service_plugin_failure (NM_VPN_SERVICE_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_LOGIN_FAILED);
 		break;
 	case 1:
 		/* Other error (couldn't bind to address, etc) */
-		nm_vpn_plugin_failure (NM_VPN_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
+		nm_vpn_service_plugin_failure (NM_VPN_SERVICE_PLUGIN (plugin), NM_VPN_PLUGIN_FAILURE_CONNECT_FAILED);
 		break;
 	default:
+		nm_vpn_service_plugin_disconnect (NM_VPN_SERVICE_PLUGIN (plugin), NULL);
 		break;
 	}
-
-	nm_vpn_plugin_set_state (NM_VPN_PLUGIN (plugin), NM_VPN_SERVICE_STATE_STOPPED);
 }
 
 static gint
-nm_openconnect_start_openconnect_binary (NMOPENCONNECTPlugin *plugin,
-                                         NMSettingVPN *s_vpn,
+nm_openconnect_start_openconnect_binary (NMOpenconnectPlugin *plugin,
+                                         NMSettingVpn *s_vpn,
                                          GError **error)
 {
-	NMOPENCONNECTPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
+	NMOpenconnectPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
 	GPid	pid;
 	const char **openconnect_binary = NULL;
 	GPtrArray *openconnect_argv;
@@ -481,14 +479,14 @@ nm_openconnect_start_openconnect_binary (NMOPENCONNECTPlugin *plugin,
 	return 0;
 }
 static gboolean
-real_connect (NMVPNPlugin   *plugin,
+real_connect (NMVpnServicePlugin   *plugin,
               NMConnection  *connection,
               GError       **error)
 {
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 	gint openconnect_fd = -1;
 
-	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
+	s_vpn = nm_connection_get_setting_vpn (connection);
 	g_assert (s_vpn);
 	if (!nm_openconnect_properties_validate (s_vpn, error))
 		goto out;
@@ -507,21 +505,21 @@ real_connect (NMVPNPlugin   *plugin,
 }
 
 static gboolean
-real_need_secrets (NMVPNPlugin *plugin,
+real_need_secrets (NMVpnServicePlugin *plugin,
                    NMConnection *connection,
-                   char **setting_name,
+                   const char **setting_name,
                    GError **error)
 {
-	NMSettingVPN *s_vpn;
+	NMSettingVpn *s_vpn;
 
-	g_return_val_if_fail (NM_IS_VPN_PLUGIN (plugin), FALSE);
+	g_return_val_if_fail (NM_IS_VPN_SERVICE_PLUGIN (plugin), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
 
-	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
+	s_vpn = nm_connection_get_setting_vpn (connection);
 	if (!s_vpn) {
 		g_set_error (error,
 		             NM_VPN_PLUGIN_ERROR,
-		             NM_VPN_PLUGIN_ERROR_CONNECTION_INVALID,
+		             NM_VPN_PLUGIN_ERROR_INVALID_CONNECTION,
 		             "%s",
 		             "Could not process the request because the VPN connection settings were invalid.");
 		return FALSE;
@@ -557,10 +555,10 @@ ensure_killed (gpointer data)
 }
 
 static gboolean
-real_disconnect (NMVPNPlugin   *plugin,
+real_disconnect (NMVpnServicePlugin   *plugin,
                  GError       **err)
 {
-	NMOPENCONNECTPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
+	NMOpenconnectPluginPrivate *priv = NM_OPENCONNECT_PLUGIN_GET_PRIVATE (plugin);
 
 	if (priv->pid) {
 		if (kill (priv->pid, SIGTERM) == 0)
@@ -576,17 +574,17 @@ real_disconnect (NMVPNPlugin   *plugin,
 }
 
 static void
-nm_openconnect_plugin_init (NMOPENCONNECTPlugin *plugin)
+nm_openconnect_plugin_init (NMOpenconnectPlugin *plugin)
 {
 }
 
 static void
-nm_openconnect_plugin_class_init (NMOPENCONNECTPluginClass *openconnect_class)
+nm_openconnect_plugin_class_init (NMOpenconnectPluginClass *openconnect_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (openconnect_class);
-	NMVPNPluginClass *parent_class = NM_VPN_PLUGIN_CLASS (openconnect_class);
+	NMVpnServicePluginClass *parent_class = NM_VPN_SERVICE_PLUGIN_CLASS (openconnect_class);
 
-	g_type_class_add_private (object_class, sizeof (NMOPENCONNECTPluginPrivate));
+	g_type_class_add_private (object_class, sizeof (NMOpenconnectPluginPrivate));
 
 	/* virtual methods */
 	parent_class->connect    = real_connect;
@@ -594,12 +592,22 @@ nm_openconnect_plugin_class_init (NMOPENCONNECTPluginClass *openconnect_class)
 	parent_class->disconnect = real_disconnect;
 }
 
-NMOPENCONNECTPlugin *
-nm_openconnect_plugin_new (void)
+NMOpenconnectPlugin *
+nm_openconnect_plugin_new (const char *bus_name)
 {
-	return (NMOPENCONNECTPlugin *) g_object_new (NM_TYPE_OPENCONNECT_PLUGIN,
-	                                             NM_VPN_PLUGIN_DBUS_SERVICE_NAME, NM_DBUS_SERVICE_OPENCONNECT,
-	                                             NULL);
+	NMOpenconnectPlugin *plugin;
+	GError *error = NULL;
+
+	plugin = (NMOpenconnectPlugin *) g_initable_new (NM_TYPE_OPENCONNECT_PLUGIN, NULL, &error,
+	                                                 NM_VPN_SERVICE_PLUGIN_DBUS_SERVICE_NAME, bus_name,
+	                                                 NM_VPN_SERVICE_PLUGIN_DBUS_WATCH_PEER, !debug,
+	                                                 NULL);
+	if (!plugin) {
+		g_warning ("Failed to initialize a plugin instance: %s", error->message);
+		g_error_free (error);
+	}
+
+	return plugin;
 }
 
 static void
@@ -624,21 +632,22 @@ setup_signals (void)
 }
 
 static void
-quit_mainloop (NMOPENCONNECTPlugin *plugin, gpointer user_data)
+quit_mainloop (NMOpenconnectPlugin *plugin, gpointer user_data)
 {
 	g_main_loop_quit ((GMainLoop *) user_data);
 }
 
 int main (int argc, char *argv[])
 {
-	NMOPENCONNECTPlugin *plugin;
-
+	NMOpenconnectPlugin *plugin;
 	gboolean persist = FALSE;
 	GOptionContext *opt_ctx = NULL;
+	gchar *bus_name = NM_DBUS_SERVICE_OPENCONNECT;
 
 	GOptionEntry options[] = {
 		{ "persist", 0, 0, G_OPTION_ARG_NONE, &persist, N_("Don't quit when VPN connection terminates"), NULL },
 		{ "debug", 0, 0, G_OPTION_ARG_NONE, &debug, N_("Enable verbose debug logging (may expose passwords)"), NULL },
+		{ "bus-name", 0, 0, G_OPTION_ARG_STRING, &bus_name, N_("D-Bus name to use for this instance"), NULL },
 		{NULL}
 	};
 
@@ -661,7 +670,8 @@ int main (int argc, char *argv[])
 	g_option_context_add_main_entries (opt_ctx, options, NULL);
 
 	g_option_context_set_summary (opt_ctx,
-		_("nm-openconnect-service provides integrated Cisco AnyConnect SSL VPN capability to NetworkManager."));
+	                              _("nm-openconnect-service provides integrated "
+	                                "Cisco AnyConnect SSL VPN capability to NetworkManager."));
 
 	g_option_context_parse (opt_ctx, &argc, &argv, NULL);
 	g_option_context_free (opt_ctx);
@@ -675,7 +685,10 @@ int main (int argc, char *argv[])
 	if (system ("/sbin/modprobe tun") == -1)
 		exit (EXIT_FAILURE);
 
-	plugin = nm_openconnect_plugin_new ();
+	if (bus_name)
+		setenv ("NM_DBUS_SERVICE_OPENCONNECT", bus_name, 0);
+
+	plugin = nm_openconnect_plugin_new (bus_name);
 	if (!plugin)
 		exit (EXIT_FAILURE);
 
